@@ -1,18 +1,23 @@
 from ConfigParser import RawConfigParser
-from deskbar.core.BrowserMatch import BrowserSmartMatch, BrowserMatch
-from deskbar.core.BrowserMatch import is_preferred_browser
+from deskbar.core.BrowserMatch import BrowserSmartMatch, BrowserMatch, is_preferred_browser, get_preferred_browser
 from deskbar.core.GconfStore import GconfStore
 from deskbar.core.Watcher import FileWatcher, DirWatcher
 from deskbar.defs import VERSION
 from gettext import gettext as _
 from os.path import join, expanduser, exists, basename
 from xml.dom import minidom
-import deskbar, deskbar.core.Indexer, deskbar.interfaces.Module
+import HTMLParser
+import base64
+import deskbar
+import deskbar.core.Indexer
+import deskbar.interfaces.Module
+import glob
 import gtk
 import logging
-import os, re, HTMLParser, base64, glob
+import os
+import re
+import subprocess
 import urllib
-
 LOGGER = logging.getLogger(__name__)
 
 # Check for presence of set to be compatible with python 2.3
@@ -21,11 +26,14 @@ try:
 except NameError:
     from sets import Set as set
 
-
 # Whether we will index firefox or mozilla bookmarks
 USING_FIREFOX = False
 if is_preferred_browser("firefox") or is_preferred_browser("iceweasel"):
     USING_FIREFOX = True
+    
+# Minimum and maximum version of Firefox
+MIN_FF_VERSION = [2, 0, 0, 0]
+MAX_FF_VERSION = [3, 0, 0, 0] # exclusively
         
 # File returned here should be checked for existence
 def get_mozilla_home_file(needed_file):    
@@ -57,6 +65,28 @@ def get_firefox_home_file(needed_file):
 
     return join(firefox_dir, path, needed_file)
 
+def get_firefox_version():
+    browser = get_preferred_browser ()
+    process = subprocess.Popen(browser + " -version", stdout=subprocess.PIPE, shell=True)
+    info = process.stdout.readline().split(" ")
+    pattern = re.compile("([0-9]+?)\.([0-9]+?)(\.([0-9]+))*")
+    version = None
+    for word in info:
+        if pattern.match(word):
+            match = pattern.match(word)
+            version = word[match.start():match.end()]
+    
+    if version != None:
+        # Convert to integers
+        version = [int(i) for i in version.split(".")]
+        # List must have 4 elements
+        if len(version) < 4:
+            while (len(version) < 4):
+                version.append(0)
+        return version
+    else:
+        return None
+
 # Whether we offer all of the browser's search engines, or only the primary
 # one (since by default Firefox seems to come with at least half a dozen)            
 GCONF_SHOW_ONLY_PRIMARY_KEY = GconfStore.GCONF_DIR + "/mozilla/show_only_primary_search"
@@ -177,15 +207,25 @@ class MozillaBookmarksHandler(deskbar.interfaces.Module):
     
     @staticmethod
     def has_requirements():
-        #    if deskbar.UNINSTALLED_DESKBAR:
-        #        return (deskbar.Handler.HANDLER_IS_HAPPY, None, None)
-            
-        if is_preferred_browser("firefox") or is_preferred_browser("mozilla"):
+        if is_preferred_browser("mozilla"):
             return True
+        elif is_preferred_browser("firefox"):
+            if MozillaBookmarksHandler.has_firefox_version():
+                return True
+            # TODO: mark as i18n
+            MozillaBookmarksHandler.INSTRUCTIONS = "Firefox version must be between 2.0.0.0 and 3.0.0.0"
+            return False
         else:
             MozillaBookmarksHandler.INSTRUCTIONS = _("Mozilla/Firefox is not your preferred browser.")
             return False
         
+    @staticmethod
+    def has_firefox_version():
+        version = get_firefox_version()
+        if version != None:
+            return (version >= MIN_FF_VERSION and version < MAX_FF_VERSION)
+        return False
+        
 
 class MozillaSearchHandler(deskbar.interfaces.Module):
     
@@ -232,6 +272,7 @@ class MozillaSearchHandler(deskbar.interfaces.Module):
         self.watcher.remove_all()
         
     def query(self, query):
+        self.set_priority_for_matches (self._smart_bookmarks)
         if SHOW_ONLY_PRIMARY and PRIMARY_SEARCH_ENGINE != None:
             for s in self._smart_bookmarks:
                 if s.get_name() == PRIMARY_SEARCH_ENGINE:
@@ -250,6 +291,12 @@ class MozillaSearchHandler(deskbar.interfaces.Module):
     @staticmethod
     def has_requirements():
         if is_preferred_browser("firefox") or is_preferred_browser("iceweasel"):
+            if is_preferred_browser("firefox") and not MozillaBookmarksHandler.has_firefox_version():
+                # TODO: mark as i18n
+                MozillaSearchHandler.INSTRUCTIONS = "Firefox version must be between 2.0.0.0 and 3.0.0.0"
+                return False
+            
+            # Correct firefox version or iceweasel
             MozillaSearchHandler.INSTRUCTIONS = _("You can customize which search engines are offered.")
             return True
         elif is_preferred_browser("mozilla"):
@@ -773,5 +820,15 @@ class MozillaHistoryHandler(deskbar.interfaces.Module):
         
     @staticmethod
     def has_requirements():
-        return MozillaBookmarksHandler.has_requirements()
+        if is_preferred_browser("mozilla"):
+            return True
+        elif is_preferred_browser("firefox"):
+            if MozillaBookmarksHandler.has_firefox_version():
+                return True
+            # TODO: mark as i18n
+            MozillaHistoryHandler.INSTRUCTIONS = "Firefox version must be between 2.0.0.0 and 3.0.0.0"
+            return False
+        else:
+            MozillaHistoryHandler.INSTRUCTIONS = _("Mozilla/Firefox is not your preferred browser.")
+            return False
 
