@@ -250,8 +250,8 @@ class PathProgramMatch(deskbar.interfaces.Match):
         self.add_action( OpenPathProgramAction(command, False), True )
         self.add_action( OpenPathProgramAction(command, True) )
         
-    def get_hash(self, text=None):
-        return text
+    def get_hash(self):
+        return self._name
         
 class ProgramsHandler(deskbar.interfaces.Module):
     
