@@ -4,7 +4,10 @@ import gobject
 import deskbar
 import deskbar.interfaces.Match
 from deskbar.ui.cuemiac.CuemiacItems import CuemiacCategory
-            
+import logging
+      
+LOGGER = logging.getLogger(__name__)
+
 # The sort function ids
 SORT_BY_CATEGORY = 1
 
@@ -37,17 +40,44 @@ class CuemiacModel (gtk.TreeStore):
         gtk.TreeStore.__init__ (self, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
         self.__categories = {}
         self.__match_hashes = {}
-        self.append_method = gtk.TreeStore.append # Alternatively gtk.TreeStore.prepend for bottom panel layout
         self.set_sort_func(SORT_BY_CATEGORY, self.__on_sort_categories)
-        self.set_sort_order(gtk.SORT_DESCENDING)
+        self.set_sort_order(gtk.SORT_ASCENDING)
     
     def set_sort_order(self, order):
+        """
+        @param order Either C{gtk.SORT_DESCENDING} or C{gtk.SORT_ASCENDING}
+        """
         self.set_sort_column_id(SORT_BY_CATEGORY, order)
+        if order == gtk.SORT_DESCENDING:
+            # Alternatively gtk.TreeStore.prepend for bottom panel layout
+            self.append_method = gtk.TreeStore.prepend
+        else:
+            self.append_method = gtk.TreeStore.append
     
-    def __compare(self, item1, item2):
+    def __compare_priorities(self, item1, item2):
+        """
+        @type item1: int
+        @type item2: int
+        """
         if (item1 > item2):
-            return 1
+            # item1 comes before item2
+            return -1
         elif (item1 < item2):
+            # item1 comes after item2
+            return 1
+        else:
+            return 0
+        
+    def __compare_alphabetically(self, word1, word2):
+        """
+        @type word1: str
+        @type word2: str
+        """
+        if (word1 > word2):
+            # word1 comes before word2
+            return 1
+        elif (word1 < word2):
+            # word1 comes after word2
             return -1
         else:
             return 0
@@ -59,17 +89,19 @@ class CuemiacModel (gtk.TreeStore):
         if match_obj1 == None or match_obj2 == None:
             return 0
         
-        if isinstance(match_obj1, CuemiacCategory):
-            return self.__compare( match_obj1.get_priority(), match_obj2.get_priority() )
+        if isinstance(match_obj1, CuemiacCategory) or isinstance(match_obj2, CuemiacCategory):
+            # Avoid that two categories with same priority get sorted alphabetically,
+            # because that would cause an exception     
+            return self.__compare_priorities( match_obj1.get_priority(), match_obj2.get_priority() )
         
         if match_obj1.get_priority() == match_obj2.get_priority():
             # Sort alphabetically
             a = treemodel[iter1][self.ACTIONS]
             b = treemodel[iter2][self.ACTIONS]
             if a != None and b != None:
-                return self.__compare( a.strip().lower(), b.strip().lower() )
+                return self.__compare_alphabetically( a.strip().lower(), b.strip().lower() )
         
-        return self.__compare( match_obj1.get_priority(), match_obj2.get_priority() )
+        return self.__compare_priorities( match_obj1.get_priority(), match_obj2.get_priority() )
     
     def __add_to_hash_iter_map(self, hash, iter):
         """
@@ -83,11 +115,18 @@ class CuemiacModel (gtk.TreeStore):
         """
         match_iter = self.__match_hashes[hash]
         match_obj = self[match_iter][self.MATCHES]
-        if len(match_obj.get_actions()) > 1 and self[match_iter][self.ACTIONS] != match_obj.get_name():
-            self.set_value(match_iter, self.ACTIONS, match_obj.get_name())
         match_obj.add_all_actions(actions)
     
     def __append_match(self, match_obj, query_string):
+        for action in match_obj.get_actions():
+            if not action.is_valid():
+                LOGGER.error("Action %r is not valid, removing it" % action)
+                match_obj.remove_action(action)
+
+        if len(match_obj.get_actions()) == 0:
+            LOGGER.error("Match %r has no actions" % match_obj)
+            return
+        
         if not self.__match_hashes.has_key(match_obj.get_hash()): 
             iter = self.__append ( query_string, match_obj )
             self.__add_to_hash_iter_map(match_obj.get_hash(), iter)
@@ -110,6 +149,8 @@ class CuemiacModel (gtk.TreeStore):
         #gtk.gdk.threads_leave()
         
     def __append (self, qstring, match_obj):
+        assert qstring != None and match_obj != None
+        
         if self.__categories.has_key (match_obj.get_category()):
             iter = self.__append_to_category (qstring, match_obj)
         else:
@@ -150,11 +191,8 @@ class CuemiacModel (gtk.TreeStore):
         return iter        
 
     def __append_match_to_iter (self, iter, qstring, match_obj):
-        if len(match_obj.get_actions()) == 1:
-            action = match_obj.get_actions()[0]
-            label = action.get_verb() % action.get_escaped_name(qstring)
-        else:
-            label = match_obj.get_name(qstring)
+        action = match_obj.get_actions()[0]
+        label = action.get_verb() % action.get_escaped_name(qstring)
         iter = self.append_method (self, iter, [qstring, match_obj, label])
         return iter
     
@@ -171,8 +209,5 @@ class CuemiacModel (gtk.TreeStore):
             
         return ( self.get_string_from_iter (self.get_iter(path1)) == self.get_string_from_iter (self.get_iter(path2)) )
 
-        
-
-
 if gtk.pygtk_version < (2,8,0):    
     gobject.type_register (CuemiacModel)
